<template>
  <div class="app-container">
    <div class="search_form">
      <div>
        <span class="search_title">客户名称/合同号：</span>
        <el-input
            v-model="searchForm.searchText"
            style="width: 240px"
            placeholder="输入客户名称/合同号搜索"
            @change="handleQuery"
            clearable
            :prefix-icon="Search"
        />
        <span class="search_title" style="margin-left: 10px">开票日期：</span>
        <el-date-picker
            style="width: 240px"
            v-model="searchForm.invoiceDate"
            value-format="YYYY-MM-DD"
            format="YYYY-MM-DD"
            type="date"
            placeholder="请选择"
            clearable
        />
        <el-button type="primary" @click="handleQuery" style="margin-left: 10px">搜索</el-button>
      </div>
      <div>
        <el-button @click="handleOut" type="primary" style="width: 100px">导出</el-button>
      </div>
    </div>
    <div class="table_list">
      <el-table :data="tableData" border v-loading="tableLoading"
                @selection-change="handleSelectionChange"
                :row-key="row => row.id"
                show-summary
                :summary-method="summarizeMainTable"
                height="calc(100vh - 18.5em)">
        <el-table-column align="center" type="selection" width="55" />
        <el-table-column align="center" label="序号" type="index" width="60" />
        <el-table-column label="销售合同号" prop="salesContractNo" show-overflow-tooltip/>
        <el-table-column label="客户合同号" prop="customerContractNo" show-overflow-tooltip/>
        <el-table-column label="客户名称" prop="customerName" show-overflow-tooltip/>
        <el-table-column label="产品大类" prop="productCategory" />
        <el-table-column label="规格型号" prop="specificationModel" />
        <el-table-column label="发票号" prop="invoiceNo" show-overflow-tooltip/>
        <el-table-column label="发票金额(元)" prop="invoiceTotal" show-overflow-tooltip :formatter="formattedNumber"/>
        <el-table-column label="税率" prop="taxRate" show-overflow-tooltip/>
        <el-table-column label="开票人" prop="invoicePerson" show-overflow-tooltip/>
        <el-table-column label="开票日期" prop="invoiceDate" show-overflow-tooltip/>
        <el-table-column label="发票" prop="invoiceFileName" show-overflow-tooltip>
          <template #default="scope">
            <span v-if="scope.row.invoiceFileName">{{ scope.row.invoiceFileName }}</span>
            <el-button v-else link type="primary" @click="handleDownload(scope.row)">上传</el-button>
          </template>
        </el-table-column>
        <el-table-column fixed="right" label="操作" min-width="60" align="center">
          <template #default="scope">
            <el-button link type="primary" size="small" @click="openForm(scope.row);">编辑</el-button>
          </template>
        </el-table-column>
      </el-table>
      <pagination v-show="total > 0" :total="total" layout="total, sizes, prev, pager, next, jumper" :page="page.current"
                  :limit="page.size" @pagination="paginationChange" />
    </div>
    <el-dialog v-model="dialogFormVisible" title="开票台账页面" width="70%" @close="closeDia">
      <el-form :model="form" label-width="140px" label-position="top" :rules="rules" ref="formRef">
        <el-row :gutter="30">
          <el-col :span="12">
            <el-form-item label="销售合同号：" prop="salesContractNo">
              <el-input v-model="form.salesContractNo" disabled></el-input>
            </el-form-item>
          </el-col>
          <el-col :span="12">
            <el-form-item label="客户名称：" prop="customerName">
              <el-input v-model="form.customerName" placeholder="自动填充" clearable disabled/>
            </el-form-item>
          </el-col>
        </el-row>
        <el-row :gutter="30">
          <el-col :span="12">
            <el-form-item label="发票号：" prop="invoiceNo">
              <el-input v-model="form.invoiceNo" placeholder="请输入" clearable/>
            </el-form-item>
          </el-col>
          <el-col :span="12">
            <el-form-item label="发票金额(元)：" prop="invoiceTotal">
              <el-input type="number" :step="0.01" v-model="form.invoiceTotal" placeholder="请输入" clearable/>
            </el-form-item>
          </el-col>
        </el-row>
        <el-row :gutter="30">
          <el-col :span="12">
            <el-form-item label="开票人：" prop="invoicePerson">
              <el-input v-model="form.invoicePerson" placeholder="请输入" clearable disabled/>
            </el-form-item>
          </el-col>
          <el-col :span="12">
            <el-form-item label="开票日期：" prop="invoiceDate">
              <el-date-picker
                  style="width: 100%"
                  v-model="form.invoiceDate"
                  value-format="YYYY-MM-DD"
                  format="YYYY-MM-DD"
                  type="date"
                  placeholder="请选择"
                  clearable
                  disabled
              />
            </el-form-item>
          </el-col>
        </el-row>
        <el-row :gutter="30">
          <el-col :span="24">
            <el-form-item label="附件材料：" prop="remark">
              <el-upload
                  v-model:file-list="fileList"
                  :action="upload.url"
                  multiple
                  ref="fileUpload"
                  auto-upload
                  :headers="upload.headers"
                  accept=".pdf"
                  :limit="1"
                  :before-upload="handleBeforeUpload"
                  :on-error="handleUploadError"
                  :on-success="handleUploadSuccess"
                  :on-remove="handleRemove"
              >
                <el-button type="primary">上传</el-button>
                <template #tip>
<!--                  文件格式支持 doc，docx，xls，xlsx，ppt，pptx，pdf，txt，xml，jpg，jpeg，png，gif，bmp，rar，zip，7z-->
                  <div class="el-upload__tip">
                    文件格式支持 pdf
                  </div>
                </template>
              </el-upload>
            </el-form-item>
          </el-col>
        </el-row>
      </el-form>
      <template #footer>
        <div class="dialog-footer">
          <el-button type="primary" @click="submitForm">确认</el-button>
          <el-button @click="closeDia">取消</el-button>
        </div>
      </template>
    </el-dialog>
    <el-dialog title="上传弹窗" width="20%" v-model="uploadModal">
      <el-row :gutter="30">
        <el-col :span="24">
          <el-form-item label="附件材料：" prop="remark">
            <el-upload
                v-model:file-list="fileList"
                :action="upload.url"
                multiple
                ref="fileUpload"
                auto-upload
                :headers="upload.headers"
                accept=".pdf"
                :limit="1"
                :on-exceed="handleExceed"
                :before-upload="handleBeforeUpload"
                :on-error="handleUploadError"
                :on-success="handleUploadSuccess"
                :on-remove="handleRemove"
            >
              <el-button type="primary">上传</el-button>
              <template #tip>
                <div class="el-upload__tip">
                  文件格式仅支持 pdf
                </div>
              </template>
            </el-upload>
          </el-form-item>
        </el-col>
      </el-row>
      <template #footer>
        <div class="dialog-footer">
          <el-button @click="commiInvoicetFile" type="primary">确认</el-button>
          <el-button  @click="uploadModal = false">取消</el-button>
        </div>
      </template>
    </el-dialog>
  </div>
</template>

<script setup>
import pagination from '@/components/PIMTable/Pagination.vue'
import { ref } from 'vue'
import {Search} from "@element-plus/icons-vue";
import {ElMessageBox } from "element-plus";
import { getToken } from "@/utils/auth"
import {
  invoiceLedgerSaveOrUpdate,
  invoiceLedgerProductInfo,
  commitFile,
  registrationProductPage
} from "../../../api/salesManagement/invoiceLedger.js";
import useUserStore from "@/store/modules/user.js";
const { proxy } = getCurrentInstance()
const tableData = ref([])
const productData = ref([])
const selectedRows = ref([])
const tableLoading = ref(false)
const page = reactive({
  current: 1,
  size: 10,
})
const total = ref(0)
const fileList = ref([])
const dialogFormVisible = ref(false)
const data = reactive({
  searchForm: {
    searchText: '',
    invoiceDate:''
  },
  form: {
    salesLedgerId: '',
    customerId: '',
    invoiceNo: '',
    invoiceTotal: '',
    taxRate: '',
    invoicePerson: '',
    invoiceDate: '',
    customerName:'',
    fileList:[]
  },
  rules: {
    salesLedgerId: [{ required: true, message: "请选择", trigger: "change" }],
    customerId: [{ required: true, message: "请输入", trigger: "blur" }],
    invoiceNo: [{ required: true, message: "请选择", trigger: "change" }],
    invoiceAmount: [{ required: true, message: "请输入", trigger: "blur" }],
    taxRate: [{ required: true, message: "请选择", trigger: "change" }],
    invoicePerson: [{ required: true, message: "请选择", trigger: "change" }],
    invoiceDate: [{ required: true, message: "请选择", trigger: "change" }],
    customerName: [{ required: true, message: "请选择", trigger: "change" }],
  }
})
const { searchForm, form, rules } = toRefs(data)
const currentId = ref('')
const userStore = useUserStore()
const upload = reactive({
  // 上传的地址
  url: import.meta.env.VITE_APP_BASE_API + "/invoiceLedger/uploadFile",
  // 设置上传的请求头部
  headers: { Authorization: "Bearer " + getToken() },
})
const matchFileType = ref(['pdf'])
const uploadModal = ref(false)
const formattedNumber = (row, column, cellValue) => {
  return parseFloat(cellValue).toFixed(2);
};
// 查询列表
/** 搜索按钮操作 */
const handleQuery = () => {
  page.current = 1
  getList()
}
const paginationChange = ({ current, limit }) => {
  page.current = current;
  page.size = limit;
  getList()
}
const getList = () => {
  tableLoading.value = true
  registrationProductPage({...searchForm.value, ...page}).then(res => {
    tableLoading.value = false
    tableData.value = res.data.records;
    total.value = res.data.total;
  })
}
// 表格选择数据
const handleSelectionChange = (selection) => {
  selectedRows.value = selection
}
// 主表合计方法
const summarizeMainTable = (param) => {
  return proxy.summarizeTable(param, ['invoiceTotal'], {
    ticketsNum: { noDecimal: true }, // 不保留小数
    futureTickets: { noDecimal: true }, // 不保留小数
  });
};
// 打开弹框
const openForm = (row) => {
  form.value = {}
  productData.value = []
  fileList.value = []
  currentId.value = row.id;

  invoiceLedgerProductInfo({id: row.id}).then(res => {
    form.value = {...res.data}
    fileList.value = res.data.fileList;
    if(!form.value.invoicePerson){
      form.value.invoicePerson = userStore.nickName
      form.value.entryDate = getCurrentDate();
    }
    if(!form.value.invoiceDate){
      form.value.invoiceDate = getCurrentDate();
    }
  })
  dialogFormVisible.value = true
}
// 上传多个文件就覆盖原来的
const handleExceed = (files) => {
  proxy.$refs["fileUpload"].clearFiles();
  const file = files[0];
  file.uid = genFileId();
  proxy.$refs["fileUpload"].handleStart(file);
};
// 上传前校检
function handleBeforeUpload(file) {
  console.log('file',file)
  // 校检文件大小
  if (file.size > 1024 * 1024 * 10) {
    proxy.$modal.msgError('上传文件大小不能超过10MB!')
    return false
  }
  // 判断文件格式是否符合
  const fileType = file.name.split('.').pop().toLowerCase();
  if(!matchFileType.value.includes(fileType)) {
    proxy.$modal.msgError('文件格式不匹配')
    return false
  }
  proxy.$modal.loading("正在上传文件，请稍候...")
  return true
}
// 上传失败
function handleUploadError(err) {
  proxy.$modal.msgError("上传文件失败")
  proxy.$modal.closeLoading()
}
// 上传成功回调
function handleUploadSuccess(res, file, uploadFiles) {
  proxy.$modal.closeLoading()
  if (res.code === 200) {
    proxy.$refs["fileUpload"].handleRemove(file)
    fileList.value.push(res.data)
    proxy.$modal.msgSuccess("上传成功")
  } else {
    proxy.$modal.msgError(res.msg)
    proxy.$refs.fileUpload.handleRemove(file)
  }
}
// 移除文件
function handleRemove (file) {
  let index = fileList.value.findIndex(item => item.url === file.url)
  if(index > -1) {
    fileList.value.splice(index, 1)
  }
}
// 提交表单
const submitForm = () => {
  proxy.$refs["formRef"].validate(valid => {
    if (valid) {
      form.value.fileList = fileList.value;
      invoiceLedgerSaveOrUpdate(form.value).then(res => {
        proxy.$modal.msgSuccess("提交成功")
        closeDia()
        getList()
      })
    }
  })
}
// 关闭弹框
const closeDia = () => {
  proxy.resetForm("formRef")
  dialogFormVisible.value = false
}
// 导出
const handleOut = () => {
  ElMessageBox.confirm(
      '选中的内容将被导出，是否确认导出？',
      '导出', {
        confirmButtonText: '确认',
        cancelButtonText: '取消',
        type: 'warning',
      }
  ).then(() => {
    proxy.download("/invoiceLedger/export", {}, '开票台账.xlsx')
  }).catch(() => {
    proxy.$modal.msg("已取消")
  })
}

// 打开附件上传弹窗
const handleDownload = (val) => {
  fileList.value = []
  uploadModal.value = true
  currentId.value = val.id
}

// 确认文件上传
const commiInvoicetFile = () => {
  const object = {
    fileList: fileList.value,
    id: currentId.value,
  }
  commitFile(object).then(res => {
    if (res.code === 200) {
      proxy.$modal.msgSuccess("提交成功")
      uploadModal.value = false
    }
    getList();
    currentId.value = ''
    fileList.value = []
  })

}

// 获取当前日期并格式化为 YYYY-MM-DD
function getCurrentDate() {
  const today = new Date();
  const year = today.getFullYear();
  const month = String(today.getMonth() + 1).padStart(2, '0'); // 月份从0开始
  const day = String(today.getDate()).padStart(2, '0');
  return `${year}-${month}-${day}`;
}

getList()
</script>

<style scoped lang="scss">

</style>
